package com.vendor.platform.core.internal;

package .var.folders.cl.9twcgjfx4vsg5zl73c6wxmvm0000gn.T;

import com.vendor.platform.api.IDeviceChangedListener;
import com.vendor.platform.api.IDeviceManager;
import com.vendor.platform.api.chips.Chip;
import com.vendor.platform.api.chips.Chips;
import com.vendor.platform.api.chips.Tool;
import com.vendor.platform.core.IToolManager;
import com.vendor.platform.core.PlatformCoreActivator;
import com.vendor.platform.core.ToolDescriptor;
import com.vendor.platform.core.internal.ErrorLog;
import com.vendor.platform.core.internal.PreferenceUtils;
import com.vendor.platform.core.internal.StringConstants;
import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.Plugin;
import org.eclipse.equinox.p2.metadata.IInstallableUnit;
import org.eclipse.ui.PlatformUI;
import org.osgi.service.prefs.BackingStoreException;
import org.osgi.service.prefs.Preferences;

public class DeviceManager implements IDeviceManager {
  private static String LIST = StringConstants.decode("1818850164");
  
  private static String CONFIG_MODEL_SCHEMA_FILE = StringConstants.decode("544905895732323196367716");
  
  private static String DEVICE_POPUP = "device.popup";
  
  private static Chips currentChips = null;
  
  private static IDeviceManager instance = new com.vendor.platform.core.internal.DeviceManager();
  
  private final List<IDeviceChangedListener> listeners = new ArrayList<>();
  
  private String DEVICE_NODE = StringConstants.decode("110386941092709");
  
  private String CURRENT_DEVICE = StringConstants.decode("27995157100326516");
  
  public static IDeviceManager getInstance() {
    return instance;
  }
  
  private static void SetChips(Chips chips) {
    currentChips = chips;
  }
  
  public void addDeviceChangedListener(IDeviceChangedListener listener) {
    synchronized (this.listeners) {
      this.listeners.add(listener);
    } 
  }
  
  public void removeDeviceChangedListener(IDeviceChangedListener listener) {
    synchronized (this.listeners) {
      Iterator<IDeviceChangedListener> iterator = this.listeners.iterator();
      while (iterator.hasNext()) {
        IDeviceChangedListener next = iterator.next();
        if (next == listener)
          iterator.remove(); 
      } 
    } 
  }
  
  public synchronized String getCurrentDevice() {
    Preferences node = getPreferences();
    try {
      if (node != null) {
        String current = node.get(this.CURRENT_DEVICE, "");
        return current;
      } 
    } catch (Exception e) {
      ErrorLog.getInstance((Plugin)PlatformCoreActivator.getDefault())
        .logError(e.getMessage(), e);
    } 
    return "";
  }
  
  public List<String> getAllDevices() {
    Preferences node = getPreferences();
    try {
      if (node != null) {
        String list = decrypt(node.get(LIST, ""));
        return Arrays.asList(list.split(";"));
      } 
    } catch (Exception e) {
      ErrorLog.getInstance((Plugin)PlatformCoreActivator.getDefault())
        .logError(e.getMessage(), e);
    } 
    return Collections.emptyList();
  }
  
  public void updateDevices(File file, boolean replace) throws Exception {
    updateDevices(file, replace, true);
  }
  
  public void updateDevices(File file, boolean replace, boolean restart) throws Exception {
    Preferences node = getPreferences();
    try {
      String res = loadFile(replace, file);
      node.put(LIST, encrypt(res));
      node.flush();
      SetChips(initChipsData());
      if (!ifContainCurrentChip() && restart) {
        resetDevicePopupIfNotSetTrue();
        setCurrentDevice("");
        PlatformUI.getWorkbench().restart();
      } else {
        setCurrentDevice(getCurrentDevice());
      } 
    } catch (Exception e) {
      throw e;
    } 
  }
  
  private String loadFile(boolean replace, File file) throws Exception {
    Chips combinedChips = new Chips();
    String rs = "";
    try {
      combinedChips.setChips(Chips.getChipList(file));
      if (!replace) {
        Chips originalChips = initChipsData();
        if (originalChips != null) {
          ArrayList<Chip> chips = originalChips.getChips();
          if (chips != null && chips.size() > 0)
            for (int i = 0; i < chips.size(); i++)
              combinedChips(combinedChips, chips.get(i));  
        } 
      } 
      rs = constructChipsPreferenceString(combinedChips);
    } catch (Exception e) {
      throw e;
    } 
    return rs;
  }
  
  private void combinedChips(Chips combinedChips, Chip newAdded) {
    if (combinedChips != null && combinedChips.getChips() != null) {
      ArrayList<Chip> chips = combinedChips.getChips();
      if (chips.size() == 0) {
        chips.add(newAdded);
        return;
      } 
      boolean duplicate = false;
      for (int i = 0; i < chips.size(); i++) {
        Chip current = chips.get(i);
        if (current.getId().equals(newAdded.getId()) && current
          .getName().equals(newAdded.getName()) && current
          .getProductionLine().equals(newAdded
            .getProductionLine())) {
          duplicate = true;
          ArrayList<Tool> newAddedToolList = newAdded.getToolList();
          if (newAddedToolList == null || newAddedToolList
            .size() == 0)
            break; 
          ArrayList<Tool> currentToolList = current.getToolList();
          if (currentToolList == null || currentToolList.size() == 0) {
            current.setToolList(newAddedToolList);
            break;
          } 
          for (int k = 0; k < newAddedToolList.size(); k++) {
            boolean find = false;
            Tool newAddedTool = newAddedToolList.get(k);
            for (int j = 0; j < currentToolList.size(); j++) {
              Tool currentTool = currentToolList.get(j);
              if (currentTool.getId()
                .equals(newAddedTool.getId()) && currentTool
                .getName().equals(newAddedTool
                  .getName())) {
                find = true;
                break;
              } 
            } 
            if (!find)
              currentToolList.add(newAddedTool); 
          } 
          break;
        } 
      } 
      if (!duplicate) {
        chips.add(newAdded);
        return;
      } 
    } 
  }
  
  private String constructChipsPreferenceString(Chips chips) {
    String rs = "";
    if (chips != null && chips.getChips().size() > 0) {
      for (int i = 0; i < chips.getChips().size(); i++) {
        Chip chip = chips.getChips().get(i);
        String chipID = chip.getId();
        String chipName = chip.getName();
        String productionLine = chip.getProductionLine();
        StringBuffer record = new StringBuffer();
        record.append(chipID);
        record.append("@");
        record.append(chipName);
        record.append("@");
        record.append(productionLine);
        ArrayList<Tool> toolList = chip.getToolList();
        if (toolList != null && toolList.size() > 0) {
          record.append(":");
          for (int j = 0; j < toolList.size(); j++) {
            Tool tool = toolList.get(j);
            String toolID = tool.getId();
            String toolName = tool.getName();
            String toolparam = (tool.getParam() == null) ? "" : tool.getParam();
            record.append(toolID);
            record.append("@");
            record.append(toolName);
            record.append("@");
            record.append(toolparam);
            record.append(":");
          } 
          record.delete(record.length() - 1, record.length());
        } 
        rs = rs + rs + ";";
      } 
      if (rs.length() > 0)
        rs = rs.substring(0, rs.length() - 1); 
    } 
    return rs;
  }
  
  private String encrypt(String str) {
    return str;
  }
  
  private String decrypt(String str) {
    return str;
  }
  
  public synchronized void setCurrentDevice(String device) {
    Preferences node = getPreferences();
    try {
      if (node != null) {
        node.put(this.CURRENT_DEVICE, device);
        node.flush();
        fireDeviceChanged(device);
      } 
    } catch (Exception e) {
      ErrorLog.getInstance((Plugin)PlatformCoreActivator.getDefault())
        .logError(e.getMessage(), e);
    } 
  }
  
  private synchronized void fireDeviceChanged(String device) {
    Chip chip = findChipByName(device);
    synchronized (this.listeners) {
      List<IDeviceChangedListener> synchronizedList = Collections.synchronizedList(this.listeners);
      for (int i = 0; i < synchronizedList.size(); i++) {
        try {
          IDeviceChangedListener next = synchronizedList.get(i);
          next.deviceChanged(chip);
        } catch (Exception e) {
          ErrorLog.getInstance((Plugin)PlatformCoreActivator.getDefault())
            .logError(e.getMessage(), e);
        } 
      } 
    } 
  }
  
  public Chip findChipByName(String device) {
    if (getCurrentChips() != null) {
      ArrayList<Chip> children = currentChips.getChips();
      if (children != null && children.size() > 0)
        for (int i = 0; i < children.size(); i++) {
          Chip chip = children.get(i);
          if (chip.getName().equals(device))
            return chip; 
        }  
    } 
    return null;
  }
  
  private Preferences getPreferences() {
    return PreferenceUtils.getPreferenceNode("com.vendor.platform.core", this.DEVICE_NODE);
  }
  
  public static File getConfigModelSchemaFile() {
    URL resource = Chips.class.getResource(CONFIG_MODEL_SCHEMA_FILE);
    if (resource == null)
      return null; 
    File schema = null;
    try {
      schema = new File(FileLocator.toFileURL(resource).toURI());
    } catch (Exception e) {
      try {
        schema = new File(FileLocator.toFileURL(resource).getFile());
      } catch (IOException e1) {
        return null;
      } 
    } 
    return schema;
  }
  
  public Chips initChipsData() {
    Chips result = new Chips();
    Preferences node = getPreferences();
    if (node != null)
      try {
        ArrayList<Chip> chips = new ArrayList<>();
        String res = decrypt(node.get(LIST, ""));
        if (res != null && res.length() > 0) {
          String[] chipArr = res.split(";");
          if (chipArr != null && chipArr.length > 0) {
            IToolManager toolManager = PlatformCoreActivator.getDefault().getToolManager();
            List<IInstallableUnit> installIUs = toolManager.getInstalledToolIUs();
            List<ToolDescriptor> featureTools = toolManager.collectFeatureTools(installIUs);
            Map<String, List<String>> map = findPluginsByToolBuildID(installIUs, featureTools);
            for (String chipDetail : chipArr) {
              Chip chip = new Chip();
              String[] detail = chipDetail.split(":");
              if (detail != null && detail.length > 0) {
                ArrayList<Tool> tools = new ArrayList<>();
                for (int i = 0; i < detail.length; i++) {
                  String content = detail[i];
                  if (i == 0) {
                    String[] idArry = content.split("@");
                    if (idArry != null && idArry.length == 3) {
                      chip.setId(idArry[0]);
                      chip.setName(idArry[1]);
                      chip.setProductionLine(idArry[2]);
                    } 
                  } else {
                    String[] idArry = content.split("@");
                    if (idArry != null) {
                      Tool tool = new Tool();
                      if (idArry.length == 2) {
                        tool.setId(idArry[0]);
                        tool.setName(idArry[1]);
                        tool.setParam("");
                      } else if (idArry.length == 3) {
                        tool.setId(idArry[0]);
                        tool.setName(idArry[1]);
                        tool.setParam(idArry[2]);
                      } 
                      tool.setContainingPlugin(map
                          .get(tool.getId()));
                      tools.add(tool);
                    } 
                  } 
                } 
                chip.setToolList(tools);
              } 
              chips.add(chip);
            } 
          } 
        } 
        result.setChips(chips);
      } catch (Exception e) {
        ErrorLog.getInstance((Plugin)PlatformCoreActivator.getDefault())
          .logError(e.getMessage(), e);
      }  
    return result;
  }
  
  private Map<String, List<String>> findPluginsByToolBuildID(List<IInstallableUnit> installIUs, List<ToolDescriptor> tools) {
    Map<String, List<String>> map = new HashMap<>();
    if (tools != null && tools.size() > 0)
      for (int i = 0; i < tools.size(); i++) {
        ToolDescriptor descriptor = tools.get(i);
        map.put((String)descriptor.getProperties().get("build.id"), Arrays.asList(descriptor.getPluginIds()));
      }  
    return map;
  }
  
  public static synchronized Chips getCurrentChips() {
    if (currentChips == null)
      currentChips = ((com.vendor.platform.core.internal.DeviceManager)instance).initChipsData(); 
    return currentChips;
  }
  
  public boolean ifContainCurrentChip() {
    if (getCurrentChips() != null && currentChips.getChips().size() > 0) {
      ArrayList<Chip> chips = currentChips.getChips();
      for (int i = 0; i < chips.size(); i++) {
        Chip chip = chips.get(i);
        if (chip.getName().equals(getCurrentDevice()))
          return true; 
      } 
    } 
    return false;
  }
  
  public void updateDevicePopup(boolean state) {
    Preferences node = getPreferences();
    try {
      if (node != null) {
        node.putBoolean(DEVICE_POPUP, state);
        node.flush();
      } 
    } catch (Exception e) {
      ErrorLog.getInstance((Plugin)PlatformCoreActivator.getDefault())
        .logError(e.getMessage(), e);
    } 
  }
  
  private void resetDevicePopupIfNotSetTrue() {
    if (!needPopupDevice()) {
      Preferences node = getPreferences();
      try {
        if (node != null) {
          node.remove(DEVICE_POPUP);
          node.flush();
        } 
      } catch (Exception e) {
        ErrorLog.getInstance((Plugin)PlatformCoreActivator.getDefault())
          .logError(e.getMessage(), e);
      } 
    } 
  }
  
  public boolean needPopupDevice() {
    Preferences node = getPreferences();
    try {
      if (node != null) {
        boolean current = node.getBoolean(DEVICE_POPUP, true);
        return current;
      } 
    } catch (Exception e) {
      ErrorLog.getInstance((Plugin)PlatformCoreActivator.getDefault())
        .logError(e.getMessage(), e);
    } 
    return true;
  }
  
  public boolean isSetPopupDevice() {
    Preferences node = getPreferences();
    try {
      if (node != null) {
        boolean current = node.getBoolean(DEVICE_POPUP, false);
        return current;
      } 
    } catch (Exception e) {
      ErrorLog.getInstance((Plugin)PlatformCoreActivator.getDefault())
        .logError(e.getMessage(), e);
    } 
    return false;
  }
  
  public void combinedNewToolSuiltableChips(String toolBuildID, String chipStr) throws BackingStoreException {
    if (chipStr != null && chipStr.length() > 0 && chipStr.indexOf(";") > 0) {
      String[] chipArr = chipStr.split(";");
      List<String> newToolSuiltableChips = Arrays.asList(chipArr);
      Chips chips = initChipsData();
      ArrayList<Chip> currentInstallChips = chips.getChips();
      boolean hasChanged = false;
      if (currentInstallChips != null && currentInstallChips.size() > 0)
        for (int i = 0; i < currentInstallChips.size(); i++) {
          boolean alreadyContain = false;
          Chip chip = currentInstallChips.get(i);
          for (int k = 0; k < newToolSuiltableChips.size(); k++) {
            String info = newToolSuiltableChips.get(k);
            if (info != null && info.length() > 0 && (info
              .split("#")).length > 1) {
              String[] split = info.split("#");
              String chipID = split[0];
              String toolBuildName = split[1];
              String toolParam = "";
              if (split.length > 2)
                toolParam = split[2]; 
              if (chipID.equals(chip.getId())) {
                ArrayList<Tool> toolList = chip.getToolList();
                for (int j = 0; j < toolList.size(); j++) {
                  Tool tool = toolList.get(j);
                  String param = (tool.getParam() == null) ? "" : tool.getParam();
                  if (tool.getId().equals(toolBuildID) && param
                    .equals(toolParam)) {
                    alreadyContain = true;
                    break;
                  } 
                } 
                if (!alreadyContain) {
                  Tool tool = new Tool();
                  tool.setId(toolBuildID);
                  tool.setName(toolBuildName);
                  tool.setParam(toolParam);
                  chip.getToolList().add(tool);
                  hasChanged = true;
                } 
              } 
            } 
          } 
        }  
      if (hasChanged) {
        Preferences node = getPreferences();
        node.put(LIST, encrypt(constructChipsPreferenceString(chips)));
        node.flush();
      } 
    } 
  }
  
  public void combinedNewTestToolSuiltableChips(String[] selectChips, String toolBuildID) throws BackingStoreException {
    if (selectChips != null && selectChips.length > 0 && toolBuildID != null && toolBuildID
      .length() > 0) {
      List<String> newToolSuiltableChips = Arrays.asList(selectChips);
      Chips chips = initChipsData();
      ArrayList<Chip> currentInstallChips = chips.getChips();
      boolean hasChanged = false;
      if (currentInstallChips != null && currentInstallChips.size() > 0)
        for (int i = 0; i < currentInstallChips.size(); i++) {
          boolean alreadyContain = false;
          Chip chip = currentInstallChips.get(i);
          for (int k = 0; k < newToolSuiltableChips.size(); k++) {
            String chipID = newToolSuiltableChips.get(k);
            if (chipID.equals(chip.getId())) {
              ArrayList<Tool> toolList = chip.getToolList();
              for (int j = 0; j < toolList.size(); j++) {
                Tool tool = toolList.get(j);
                if (tool.getId().equals(toolBuildID)) {
                  alreadyContain = true;
                  break;
                } 
              } 
              if (!alreadyContain) {
                Tool tool = new Tool();
                tool.setId(toolBuildID);
                tool.setName(toolBuildID);
                chip.getToolList().add(tool);
                hasChanged = true;
              } 
            } 
          } 
        }  
      if (hasChanged) {
        Preferences node = getPreferences();
        node.put(LIST, encrypt(constructChipsPreferenceString(chips)));
        node.flush();
      } 
    } 
  }
}
